#include <JGL/types/Light.h>

JGL::PointLight::PointLight(const Vector3& position, const Color4& ambient, const Color4& diffuse, const Color4& specular, float constant_attenuation, float linear_attenuation, float quadratic_attenuation) {
    this->position = Vector4(position, 1.0f);
    this->ambient = ambient;
    this->diffuse = diffuse;
    this->specular = specular;
    this->constant_attenuation = constant_attenuation;
    this->linear_attenuation = linear_attenuation;
    this->quadratic_attenuation = quadratic_attenuation;
}

float JGL::PointLight::GetAttenuationAtPosition(const Vector3& pos) const {
    Vector3 light_pos = {position.x, position.y, position.z};
    Vector3 vector_to_position = pos - light_pos;
    float distance = vector_to_position.Length();

    return 1.0f / (GetConstantAttenuation() + GetLinearAttenuation() * distance + GetQuadraticAttenuation() * distance * distance);
}

JGL::SpotLight::SpotLight(const Vector3& position, const Matrix3x3& ro_mat, float cone_size_degrees, float exponent, const Color4& ambient, const Color4& diffuse, const Color4& specular,
                          float constant_attenuation, float linear_attenuation, float quadratic_attenuation) {
    this->position = Vector4(position, 1);
    //TODO RotationMatrix to "Normalized direction vector."
    orientation = ro_mat;
    this->cut = cone_size_degrees;
    this->exponent = exponent;
    this->ambient = ambient;
    this->diffuse = diffuse;
    this->specular = specular;
    this->constant_attenuation = constant_attenuation;
    this->linear_attenuation = linear_attenuation;
    this->quadratic_attenuation = quadratic_attenuation;
}

Vector4 JGL::LightBase::GetPosition() const {
    return position;
}

Color4 JGL::LightBase::GetAmbient() const {
    return ambient;
}

Color4 JGL::LightBase::GetDiffuse() const {
    return diffuse;
}

Color4 JGL::LightBase::GetSpecular() const {
    return specular;
}

float JGL::LightBase::GetConstantAttenuation() const {
    return constant_attenuation;
}

float JGL::LightBase::GetLinearAttenuation() const {
    return linear_attenuation;
}

float JGL::LightBase::GetQuadraticAttenuation() const {
    return quadratic_attenuation;
}


